@@ -1,4 +1,11 @@
-__version__ = '2.0.6'
+__version__ = '2.1.0'
+
+try:
+    import listeners
+except ImportError:
+    # this happens when setup.py is grabbing __version__; nothing to worry
+    # about
+    pass
 
 """
 The Pygments reStructuredText directive
